/***************************************************************************
 *   Copyright (C) 2006 by Fabrice DUCOS   *
 *   fabrice.ducos@icare.univ-lille1.fr   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "Hdf_file.hpp"

int main(int argc, char* argv[]) {

  cout<<"***** TEST FILE OPENING *****"<<endl;
  // Unexisting file
  try {
    Hdf_file f("/dummy.hdf",DFACC_READ);
  } catch (bad_hdf_file &e) {
    cout<<"CATCHED "<<e.what()<<endl;
  }
 Hdf_file f("",DFACC_READ);
  cout<<"***** TEST VDATA INITIALIZATION *****"<<endl;
  // read the VData general atributes
  for (int i = 0 ; i<f.get_n_vd() ; ++i)
    cout<<f.get_vd(i).to_string();

  cout<<"***** TEST READING THE DATAS IN A VDATA *****"<<endl;
  string vd_name;
  string field_name;
  // Wrong VData
  vd_name="Dummy";
  try {
    Hdf_vd vd = f.get_vd(vd_name.c_str());
    } catch ( bad_vd_name &e ) {
    cout<<"CATCHED : "<<e.what()<<endl;
  }
  // Wrong VData Field
  vd_name="Latitude";
  field_name="Dummy";
  try {
    Hdf_vd vd = f.get_vd(vd_name.c_str());
    Hdf_vd_field vd_field=vd.get_field(field_name.c_str()); // access to one field in one VData
    } catch ( bad_field_name &e ) {
    cout<<"CATCHED : "<<e.what()<<endl;
  }
  // valid VData and field -> read the datas
  field_name="Latitude";
  Hdf_vd vd = f.get_vd(vd_name.c_str()); // access to one VData using the name
  Hdf_vd_field vd_field=vd.get_field(field_name.c_str()); // access to one field in one VData
  // read the data of this field
  int32 nb_record = vd.get_nb_record(); // number of entries in the VData
  float32 data[nb_record]; // buffer to fill
  f.read_vdata_field(vd_name.c_str(),field_name.c_str(),data);
  // print out the read datas
  cout<<"VData: "<<vd_name<<" Field: "<<field_name<<endl;
  cout<<"index\tValue"<<endl;
  for (int i=0;i<nb_record;++i)
    cout<<i<<"\t"<<data[i]<<endl;

  cout<<"***** TEST OPEN DIFFERENT FILE TYPES *****"<<endl;
// 
   cout<<"--- CALIPSO ---"<<endl;
   Hdf_file f1=Hdf_file("/home/pascal/DATA/CALIPSO/CAL_LID_L2_01kmCLay/2004/2004_01_01/CAL_LID_L2_01kmCLay-BETA-V0-00.2004-01-01T00-20-11ZN.hdf",DFACC_READ);

  cout<<"--- MODIS ---"<<endl;
  Hdf_file f2=Hdf_file("/home/pascal/DATA/MODIS/MYD06_L2/2004/2004_01_01/MYD06_L2.A2004001.0000.004.2004077230457.hdf",DFACC_READ);
  cout<<f2<<endl;

  cout<<"--- CERES ---"<<endl;
  Hdf_file f3=Hdf_file("/home/pascal/DATA/CERES/CRS/2004/2004_01_01/CER_CRS_Aqua-FM4-MODIS_Beta1_018025.2004010100",DFACC_READ);
  cout<<f3<<endl;

  return 0;
}
